# -*- coding: utf-8 -*-
#
# Copyright 2017 Gehirn Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ghoauth.oauth2.errors import (
    InvalidGrantError,
    InvalidRequestError,
    ServerError,
)
from ghoauth.oauth2.token import (
    authorization_code_factory,
    bearer_token_factory,
)
from ghoauth.uriutils import extend_uri_with_params

from .abc import (
    AbstractBaseGrantTypeHandler,
    AbstractBaseResponseTypeHandler,
)


class AuthorizationCodeGrant(AbstractBaseResponseTypeHandler,
                             AbstractBaseGrantTypeHandler):
    """
    https://tools.ietf.org/html/rfc6749#section-4.1
    """
    __slots__ = ['code_factory', 'token_factory']

    def __init__(self,
                 code_factory=authorization_code_factory,
                 token_factory=bearer_token_factory):
        self.code_factory = code_factory
        self.token_factory = token_factory

    def supported_response_types(self):
        return ['code']

    def get_default_response_mode(self, request):
        return 'query'

    def is_response_mode_allowed(self, response_mode, request):
        return response_mode == 'query'

    def handle_authorization_request(self, request, repository, subject):
        try:
            code = self.code_factory(subject, request.o2_scope,
                                     request.o2_client_id, request)
            if request.o2_state:
                code['state'] = request.o2_state
            repository.save_authorization_code(code, request.o2_redirect_uri,
                                               request.o2_client_id, request)
        except BaseException as why:
            raise ServerError(request) from why

        redirect_uri = extend_uri_with_params(request.o2_redirect_uri, code,
                                              request.o2_response_mode)
        request.response.status_code = 302
        request.response.location = redirect_uri
        return None

    def validate_token_request(self, request, repository, validator):
        super().validate_token_request(request, repository, validator)

        if not request.o2_validate(requireds={'code'},
                                   optionals={'redirect_uri'}):
            raise InvalidRequestError(request)

        if not request.o2_validate_redirect_uri(repository, validator):
            raise InvalidGrantError(request)

        if not validator.validate_authorization_code(
                request.o2_code, request.o2_redirect_uri,
                request.o2_client_id, request):
            raise InvalidGrantError(request)

    def supported_grant_types(self):
        return ['authorization_code']

    def handle_token_request(self, request, repository):
        try:
            subject = repository.get_subject_by_authorization_code(
                request.o2_code, request)
            scope = repository.get_scope_by_authorization_code(
                request.o2_code, request)

            token = self.token_factory(subject, scope, True, request)
            repository.save_access_token(token, request)
        except BaseException as why:
            raise ServerError(request) from why

        return token



# -*- coding: utf-8 -*-
#
# Copyright 2017 Gehirn Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class AbstractBaseRepository:

    def is_client_available(self, client_id, request):
        raise NotImplementedError('must implement is_client_available')

    def get_primary_redirect_uri(self, client_id, request):
        raise NotImplementedError('must implement get_primary_redirect_uri')

    def get_default_scope(self, request):
        raise NotImplementedError('must implement get_default_scope')

    def save_access_token(self, token, request):
        raise NotImplementedError('must implement save_access_token')


class AuthorizationCodeGrantRepositoryMixin:

    def get_subject_by_authorization_code(self, code, request):
        raise NotImplementedError('must implement get_subject_by_authorization_code')  # noqa

    def get_scope_by_authorization_code(self, code, request):
        raise NotImplementedError('must implement get_scope_by_authorization_code')  # noqa

    def save_authorization_code(self, code, redirect_uri, client_id, request):
        raise NotImplementedError('must implement save_authorization_code')


class RefreshTokenGrantRepositoryMixin:

    def get_subject_by_refresh_token(self, refresh_token, request):
        raise NotImplementedError('must implement get_subject_by_refresh_token')  # noqa

    def get_scope_by_refresh_token(self, refresh_token, request):
        raise NotImplementedError('must implement get_scope_by_refresh_token')


class JWTBearerGrantRepositoryMixin:

    def get_key_by_jwt_headers(self, headers, client_id, request):
        raise NotImplementedError('must implement get_key_by_jwt_headers')

    def get_subject_by_jwt_claims(self, jwt_claims, request):
        raise NotImplementedError('must implement get_subject_by_jwt_claims')


class OpenIDRepositoryMixin:

    def get_id_token_by_subject(self, subject, code, token, request):
        raise NotImplementedError('must implement get_id_token_by_subject')



# -*- coding: utf-8 -*-
#
# Copyright 2017 Gehirn Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ghoauth.oauth2.errors import (
    AccessDeniedError,
    FatalOAuth2Error,
    InvalidRequestError,
    UnsupportedResponseTypeError,
)
from ghoauth.oauth2.validator import normalize_response_type

from .abc import AbstractBaseEndpoint


class AuthorizationEndpoint(AbstractBaseEndpoint):
    __slots__ = ['repository', 'validator', '_handlers']

    def _get_handler(self, request):
        """
        If an authorization request is missing the "response_type" parameter,
        or if the response type is not understood, the authorization server
        MUST return an error response as described in Section 4.1.2.1.
        https://tools.ietf.org/html/rfc6749#section-3.1.1
        """
        try:
            return super()._get_handler(request.o2_response_type)
        except KeyError:
            if not request.o2_validate(requireds={'client_id'},
                                       optionals={'redirect_uri'}):
                raise FatalOAuth2Error()
            if not self.repository.is_client_available(
                    request.o2_client_id, request):
                raise FatalOAuth2Error()
            if not request.o2_validate_redirect_uri(
                    self.repository, self.validator):
                raise FatalOAuth2Error()

            raise UnsupportedResponseTypeError(request)

    def register_handler(self, handler):
        # TODO(yosida95): validate the response_type
        response_types = (normalize_response_type(v)
                          for v in handler.supported_response_types())
        super().register_handler(handler, response_types)

    def validate_request(self, request):
        handler = self._get_handler(request)

        if not request.o2_response_mode:
            request.o2_response_mode = \
                handler.get_default_response_mode(request)
        if not handler.is_response_mode_allowed(
                request.o2_response_mode, request):
            raise InvalidRequestError(request)

        handler.validate_authorization_request(
            request, self.repository, self.validator)

    def handle_request(self, request, subject, is_authorized):
        self.validate_request(request)
        if not is_authorized:
            raise AccessDeniedError(request)

        handler = self._get_handler(request)
        handler.handle_authorization_request(request, self.repository, subject)
        request.response.cache_control.no_store = True
        request.response.pragma = 'no-cache'
        return request.response



# -*- coding: utf-8 -*-
#
# Copyright 2017 Gehirn Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import datetime

from jose import (
    JWTError,
    jwt,
)

from ghoauth.oauth2.errors import (
    InvalidGrantError,
    InvalidRequestError,
    InvalidScopeError,
    ServerError,
)
from ghoauth.oauth2.token import bearer_token_factory

from .abc import AbstractBaseGrantTypeHandler


def decode_jwt_assertion(jwt_message, decode_key, request,
                         validator, current_ts, is_authorization_grant):
    try:
        options = {'verify_signature': True,
                   'verify_iss': False,
                   'verify_sub': False,
                   'verify_aud': False,
                   'verify_exp': False,
                   'verify_nbf': False,
                   'verify_iat': False,
                   'verify_jti': False}
        jwt_claims = jwt.decode(jwt_message, decode_key, options=options)
    except JWTError:
        return None

    iss = jwt_claims.get('iss')
    if not isinstance(iss, str) \
            or not validator.validate_iss(iss, request):
        return None
    sub = jwt_claims.get('sub')
    if not isinstance(sub, str) \
            or is_authorization_grant \
            and not validator.validate_sub(sub, request):
        return None
    aud = jwt_claims.get('aud')
    if not isinstance(aud, str) \
            or not validator.validate_aud(aud, request):
        return None
    exp = jwt_claims.get('exp')
    if not isinstance(exp, int) or exp < current_ts:
        return None
    if 'nbf' in jwt_claims:
        nbf = jwt_claims['nbf']
        if not isinstance(nbf, int) or nbf > current_ts:
            return None
    if 'iat' in jwt_claims and not isinstance(jwt_claims['iat'], int):
        return None
    if 'jti' in jwt_claims \
            and not validator.validate_jti(jwt_claims['jti'], request):
        return None

    return jwt_claims


class JWTBearerGrant(AbstractBaseGrantTypeHandler):
    """
    https://tools.ietf.org/html/rfc7523
    """
    __slots__ = ['token_factory', 'datetime_factory']

    def __init__(self, token_factory=bearer_token_factory,
                 datetime_factory=datetime.now):
        self.token_factory = token_factory
        self.datetime_factory = datetime_factory

    def supported_grant_types(self):
        return ['urn:ietf:params:oauth:grant-type:jwt-bearer']

    def validate_token_request(self, request, repository, validator):
        if not request.o2_validate(requireds={'assertion'},
                                   optionals={'scope'}):
            raise InvalidRequestError(request)

        try:
            headers = jwt.get_unverified_headers(request.o2_assertion)
            decode_key = repository.get_key_by_jwt_headers(
                headers, request.o2_client_id, request)
        except JWTError as why:
            raise InvalidGrantError(request) from why
        else:
            request._o2_jwt_claims = decode_jwt_assertion(
                request.o2_assertion, decode_key, request,
                validator, self.datetime_factory().timestamp(), True)
            if not request._o2_jwt_claims:
                raise InvalidGrantError(request)

        if request.o2_scope:
            if not validator.validate_scope(request.o2_scope, request):
                raise InvalidScopeError(request)
        else:
            request.o2_scope = repository.get_default_scope(request)

    def handle_token_request(self, request, repository):
        try:
            subject = repository.get_subject_by_jwt_claims(
                request._o2_jwt_claims, request)
            token = self.token_factory(subject, request.o2_scope,
                                       False, request)
            repository.save_access_token(token, request)
        except BaseException as why:  # pragma: no cover
            raise ServerError(request) from why

        request.response.status = 200
        return token



# -*- coding: utf-8 -*-
#
# Copyright 2017 Gehirn Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .abc import (
    AbstractBaseGrantTypeHandler,
    AbstractBaseResponseTypeHandler,
)
from .authorization_code import AuthorizationCodeGrant
from .implicit import ImplicitGrant
from .jwt_bearer import JWTBearerGrant
from .multiple_valued import MultipleValuedResponseType
from .refresh_token import RefreshTokenGrant


__all__ = ['AbstractBaseGrantTypeHandler',
           'AbstractBaseResponseTypeHandler',
           'AuthorizationCodeGrant',
           'ImplicitGrant',
           'JWTBearerGrant',
           'MultipleValuedResponseType',
           'RefreshTokenGrant']



# -*- coding: utf-8 -*-
#
# Copyright 2017 Gehirn Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .endpoints import (
    AuthorizationEndpoint,
    TokenEndpoint,
)
from .errors import OAuth2ErrorResponse
from .grant_types import (
    AbstractBaseGrantTypeHandler,
    AbstractBaseResponseTypeHandler,
)


class AuthorizationServer:

    def __init__(self, repository, validator):
        self._authz_endpoint = AuthorizationEndpoint(repository, validator)
        self._token_endpoint = TokenEndpoint(repository, validator)

    def register_handler(self, handler):
        if isinstance(handler, AbstractBaseResponseTypeHandler):
            self._authz_endpoint.register_handler(handler)
        if isinstance(handler, AbstractBaseGrantTypeHandler):
            self._token_endpoint.register_handler(handler)

    def validate_authorization_request(self, request):
        try:
            self._authz_endpoint.validate_request(request)
        except OAuth2ErrorResponse as why:
            why.o2_use_redirect()
            raise

    def validate_token_request(self, request):
        try:
            self._token_endpoint.validate_request(request)
        except OAuth2ErrorResponse as why:
            why.o2_use_json()
            raise

    def handle_authorization_request(self, request, subject, is_authorized):
        try:
            return self._authz_endpoint.handle_request(
                request, subject, is_authorized)
        except OAuth2ErrorResponse as why:
            why.o2_use_redirect()
            raise

    def handle_token_request(self, request):
        try:
            return self._token_endpoint.handle_request(request)
        except OAuth2ErrorResponse as why:
            why.o2_use_json()
            raise



# -*- coding: utf-8 -*-
#
# Copyright 2017 Gehirn Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from webob import BaseRequest

from .validator import (
    normalize_response_type,
    validate_redirect_uri,
)


class o2_property:

    def __init__(self, name):
        self.__name__ = name

    def __get__(self, obj, type_=None):
        if obj is None:  # pragma: no cover
            return self
        paramdct = obj.environ.setdefault('oauth2.params', {})
        if self.__name__ in paramdct:
            return paramdct[self.__name__]
        return None

    def __set__(self, obj, value):
        paramdct = obj.environ.setdefault('oauth2.params', {})
        paramdct[self.__name__] = value

    def __delete__(self, obj):
        raise AttributeError('can\'t delete attribute')


class request_parameter(o2_property):

    def __init__(self, name, normalizer=None):
        super().__init__(name)
        self._normalizer = normalizer or (lambda v: v)

    def __get__(self, obj, type_=None):
        value = super().__get__(obj, type_)
        paramdct = obj.environ['oauth2.params']
        if value is not None or self.__name__ in paramdct:
            return value

        values = obj.params.getall(self.__name__)
        if len(values) == 1 and values[0]:
            value = self._normalizer(values[0])
            paramdct[self.__name__] = value
            return value
        paramdct[self.__name__] = None
        return None

    def __set__(self, obj, value):
        value = self._normalizer(value)
        super().__set__(obj, value)


class OAuth2RequestMixin:
    o2_client_id = request_parameter('client_id')
    o2_client_secret = request_parameter('client_secret')
    o2_code = request_parameter('code')
    o2_grant_type = request_parameter('grant_type')
    o2_redirect_uri = request_parameter('redirect_uri')
    o2_refresh_token = request_parameter('refresh_token')
    o2_response_type = request_parameter('response_type',
                                         normalize_response_type)
    o2_scope = request_parameter('scope')
    o2_state = request_parameter('state')
    # Multiple Values Response Types
    o2_response_mode = request_parameter('response_mode')
    # Assertion Framework for OAuth 2.0 Client Authentication
    # and Authorization Grants
    o2_assertion = request_parameter('assertion')
    _o2_jwt_claims = o2_property('_o2_jwt_claims')

    def o2_validate(self, requireds=None, optionals=None):
        if requireds:
            for parameter in requireds:
                if parameter not in self.params \
                        or len(self.params.getall(parameter)) != 1 \
                        or not self.params[parameter]:
                    return False
        if optionals:
            for parameter in optionals:
                if parameter in self.params \
                        and len(self.params.getall(parameter)) != 1:
                    return False
        return True

    def o2_validate_redirect_uri(self, repository, validator, exactly=False):
        if self.o2_redirect_uri:
            if not validate_redirect_uri(self.o2_redirect_uri):
                return False
            return validator.validate_redirect_uri(
                self.o2_redirect_uri, self.o2_client_id, self, exactly)
        self.o2_redirect_uri = repository.get_primary_redirect_uri(
            self.o2_client_id, self)
        return validate_redirect_uri(self.o2_redirect_uri)


class OAuth2Request(OAuth2RequestMixin, BaseRequest):
    pass



# -*- coding: utf-8 -*-
#
# Copyright 2017 Gehirn Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import TestCase

from webob import BaseRequest

from ghoauth.oauth2.repository import AbstractBaseRepository
from ghoauth.oauth2.request import (
    OAuth2Request,
    request_parameter,
)
from ghoauth.oauth2.validator import AbstractBaseValidator


class RequestParameterTest(TestCase):

    class Request(BaseRequest):
        o2_param = request_parameter('param')
        o2_dup = request_parameter('dup')

    def test_it(self):
        req = self.Request.blank('https://example.com/?param=value&dup=1&dup=2')  # noqa
        self.assertNotIn('oauth2.params', req.environ)

        self.assertEqual(req.o2_param, 'value')
        self.assertIsNone(req.o2_dup)

        self.assertIn('oauth2.params', req.environ)
        self.assertEqual(req.environ['oauth2.params'],
                         {'param': 'value',
                          'dup': None})

        req.o2_param = 'new_value'
        self.assertEqual(req.o2_param, 'new_value')
        self.assertIsNone(req.o2_dup)
        self.assertEqual(req.environ['oauth2.params'],
                         {'param': 'new_value',
                          'dup': None})

        with self.assertRaises(AttributeError):
            del req.o2_dup


class OAuth2RequestTest(TestCase):

    class StubRepository(AbstractBaseRepository):

        def is_client_available(self, client_id, request):
            return client_id == 'yosida95'

        def get_primary_redirect_uri(self, client_id, request):
            if client_id == 'yosida95':
                return 'https://yosida95.com/cb'
            return '/cb'

    class StubValidator(AbstractBaseValidator):

        def authenticate_client(self, request):
            return 'yosida95'

        def authenticate_client_as(self, client_id, request):
            return client_id == 'yosida95'

        def validate_redirect_uri(self, redirect_uri, client_id, request,
                                  exactly=False):
            return redirect_uri == 'https://yosida95.com/cb'

    def test_o2_validate(self):
        req = OAuth2Request.blank('https://example.com/?client_id=yosida95&response_type=code&scope=abc&scope=def')  # noqa

        self.assertEqual(req.o2_client_id, 'yosida95')
        self.assertEqual(req.o2_response_type, 'code')

        # requireds
        self.assertTrue(req.o2_validate(requireds={'client_id',
                                                   'response_type'}))
        # optionals
        self.assertTrue(req.o2_validate(optionals={'redirect_uri'}))
        # mix
        self.assertTrue(req.o2_validate(requireds={'client_id',
                                                   'response_type'},
                                        optionals={'redirect_uri'}))

        # a missing parameter
        self.assertFalse(req.o2_validate(requireds={'redirect_uri'}))
        # a duplicated parameter
        self.assertFalse(req.o2_validate(requireds={'scope'}))
        self.assertFalse(req.o2_validate(optionals={'scope'}))

    def test_o2_validate_redirect_uri(self):
        repo = self.StubRepository()
        valid = self.StubValidator()

        # use valid primary
        req = OAuth2Request.blank('https://example.com/?client_id=yosida95')
        self.assertTrue(req.o2_validate_redirect_uri(repo, valid))
        self.assertEqual(req.o2_redirect_uri, 'https://yosida95.com/cb')

        # use invalid primary
        req = OAuth2Request.blank('https://example.com/?client_id=yoshida95')
        self.assertFalse(req.o2_validate_redirect_uri(repo, valid))
        self.assertEqual(req.o2_redirect_uri, '/cb')

        # use query parameters with a valid value
        req = OAuth2Request.blank('https://example.com/?client_id=yosida95&redirect_uri=https://yosida95.com/cb')  # noqa
        self.assertTrue(req.o2_validate_redirect_uri(repo, valid))
        self.assertEqual(req.o2_redirect_uri, 'https://yosida95.com/cb')

        # use query parameters with a invalid value
        req = OAuth2Request.blank('https://example.com/?client_id=yosida95&redirect_uri=/cb')  # noqa
        self.assertFalse(req.o2_validate_redirect_uri(repo, valid))
        self.assertEqual(req.o2_redirect_uri, '/cb')



# -*- coding: utf-8 -*-
#
# Copyright 2017 Gehirn Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from setuptools import (
    find_packages,
    setup,
)

here = os.path.dirname(__file__)
requires = [
    'python-jose[cryptography] ~= 3.0',  # May 4, 2018
    'rfc3986 ~= 1.4',  # Apr 8, 2020
    'WebOb ~= 1.8',  # Apr 5, 2018
]

setup(
    name='ghoauth',
    version='0.7.0',

    description='OAuth Provider Library for Python 3',
    url='https://github.com/GehirnInc/ghoauth',

    author='Kohei YOSHIDA',
    author_email='kohei.yoshida@gehirn.co.jp',

    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Security',
    ],

    packages=find_packages(),

    install_requires=requires,
    tests_require=[],
    python_requires='>= 3.6',
)



# -*- coding: utf-8 -*-
#
# Copyright 2017 Gehirn Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from collections import OrderedDict
from unittest import TestCase

from ghoauth.uriutils import (
    extend_uri_with_params,
    is_absolute_uri,
)


class extend_uri_with_params_test(TestCase):

    def test_it(self):
        d = OrderedDict()
        d['param_1'] = 'value_1'
        d['param_2'] = 'value_2'

        # query mode
        self.assertEqual(
            extend_uri_with_params('https://example.com/cb', d, 'query'),
            'https://example.com/cb?param_1=value_1&param_2=value_2')
        # redirect_uri with queries
        self.assertEqual(
            extend_uri_with_params('https://example.com/cb?k=v', d, 'query'),
            'https://example.com/cb?k=v&param_1=value_1&param_2=value_2')

        # fragment mode
        self.assertEqual(
            extend_uri_with_params('https://example.com/cb', d, 'fragment'),
            'https://example.com/cb#param_1=value_1&param_2=value_2')


class is_absolute_uri_test(TestCase):

    def test_it(self):
        self.assertTrue(is_absolute_uri('https://example.com/cb'))
        self.assertFalse(is_absolute_uri('urn:example:animal:ferret:nose'))
        self.assertFalse(is_absolute_uri('/cb'))



# -*- coding: utf-8 -*-
#
# Copyright 2017 Gehirn Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ghoauth.oauth2.errors import (
    FatalOAuth2Error,
    InvalidRequestError,
    InvalidScopeError,
)


class AbstractBaseResponseTypeHandler:

    def supported_response_types(self):
        return []

    def get_default_response_mode(self, request):
        raise NotImplementedError('must implement get_default_response_mode')

    def is_response_mode_allowed(self, response_mode, request):
        raise NotImplementedError('must implement is_response_mode_allowed')

    def validate_authorization_request(self, request, repository, validator):
        if not request.o2_validate(requireds={'client_id'},
                                   optionals={'redirect_uri'}):
            raise FatalOAuth2Error()
        if not repository.is_client_available(request.o2_client_id, request):
            raise FatalOAuth2Error()
        if not request.o2_validate_redirect_uri(repository, validator):
            raise FatalOAuth2Error()

        if not request.o2_validate(requireds={'response_type'},
                                   optionals={'scope', 'state',
                                              'response_mode'}):
            raise InvalidRequestError(request)

        if request.o2_scope:
            if not validator.validate_scope(
                    request.o2_scope, request.o2_client_id, request):
                raise InvalidScopeError(request)
        else:
            request.o2_scope = repository.get_default_scope(request)

    def handle_authorization_request(self, request, repository, subject):
        raise NotImplementedError('must implement handle_authorization_request')  # noqa


class AbstractBaseGrantTypeHandler:

    def supported_grant_types(self):
        return []

    def validate_token_request(self, request, repository, validator):
        return

    def handle_token_request(self, request, repository):
        raise NotImplementedError('must implement handle_token_request')
